﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace howto_unordered_sequence_equal
{
    public static class EnumerableExtensions
    {
        // Return true if the lists contain the
        // same elements, possibly in different orders.
        public static bool HashableSequenceEqual<T>(this IEnumerable<T> list1, IEnumerable<T> list2)
        {
            // Order the items in each list.
            var ordered1 =
                from T item in list1
                orderby item.GetHashCode()
                select item;
            var ordered2 =
                from T item in list2
                orderby item.GetHashCode()
                select item;

            // Compare the lists.
            return ordered1.SequenceEqual(ordered2);
        }

        // Return true if the lists contain the
        // same elements, possibly in different orders.
        public static bool ComparableSequenceEqual<T>(
            this IEnumerable<T> list1, IEnumerable<T> list2)
            where T : IComparable<T>
        {
            // Order the items in each list.
            var ordered1 =
                from T item in list1
                orderby item
                select item;
            var ordered2 =
                from T item in list2
                orderby item
                select item;

            // Compare the lists.
            return ordered1.SequenceEqual(ordered2);
        }
    }
}
